/*
	ImageLab needs to include a menu item to choose a directory from which 
	to make Image Filters available
*/
package imagelab;
import imagelab.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.FileSystemView;
import java.util.Vector;
import java.io.*;

/** 
*	ImageLab is a platform for image filter development.  ImageLab
*	begins by building a menu of all available filters (those .class files
*	that implement the ImageFilter interface @see ImageFilter).
@author Aaron Gordon
@version 0.2 October 21, 2004
*/
public class ImageLab {
	JFrame frame;		//the apps main frame
	JFileChooser saver;	//for saving images
	Vector<Object> images = new Vector<Object>();	//holds all open images (imgProvider objects)
	String filterDir;	//the directory where filters live - unused right now
	Vector<Object>  filters;	//holds filter objects
	ImgProvider impro;	//the current image provider
	ImageLab theLab;	//holds a copy of this

	public static void main(String [] arg) {
		ImageLab ilab = new ImageLab();
	}//main

	public ImageLab() {					//constructor
		//filterDir = "filters";		//default, build in a way for user to change
		theLab 	= this;
		filters = new Vector<Object>(10);
		frame	= new JFrame();
		Container cpane = frame.getContentPane();
		
		File parDir = new File(".");			//set to current directory
		String imdir = ".";
		try {
			imdir = parDir.getCanonicalPath() +  "/images";
			//System.out.println("\n\nParent directory:  " + imdir);
		} catch (Exception excer) {
		}
		saver	= new JFileChooser(imdir);
		JMenuBar menubar = buildMenus();
		frame.setJMenuBar(menubar);
		frame.setBounds(700,100,200,150);
		frame.setVisible(true);
	}//constructor

	private JMenuBar buildMenus() {
		JMenuBar mbar = new JMenuBar();
		JMenu file = new JMenu("File");
		mbar.add(file);
			JMenuItem open = new JMenuItem("open",'o');
			file.add(open);
			open.addActionListener(makeOpenListener());
			JMenuItem save = new JMenuItem("save",'s');
			file.add(save);
			save.addActionListener(makeSaveListener());
			JMenuItem quit = new JMenuItem("quit",'q');
			file.add(quit);
			quit.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			});
		JMenu filter = new JMenu("Filter");
		mbar.add(filter);
				//find filters and build corresponding menu items
				//look at each .class file in filterDir
				//do the classForName stuff and enter into filter menu
				//System.out.println("**********finding filters***********");
				FileSystemView fsv = FileSystemView.getFileSystemView();
				//File [] fil = fsv.getFiles(new File("."),true);
				File [] fil = fsv.getFiles(new File("filters"),true);
				//System.out.println("Found " + fil.length + " possible filters");
				String clName = " ";		//holds name of class
				for (int k=0; k<fil.length; k++) {
					if (fil[k].getName().endsWith("class")) {
						Class cl;
						Object obj;
						try {
							clName = fil[k].getName();
							int spot = clName.lastIndexOf(".");
							//clName = clName.substring(0,spot);
							clName = "filters." + clName.substring(0,spot);
							//System.out.println("Trying: " + clName);
							cl = Class.forName(clName);
							//System.out.println("Class for name is: " + cl);
							Class [] interfaces = cl.getInterfaces();
							//System.out.println("Number of interfaces is " + interfaces.length);
							boolean isFilter = false;
							for (int j=0; j<interfaces.length; j++) {
								//System.out.println("------->>>>>>>>>>" + interfaces[j].getName());
								isFilter |= interfaces[j].getName().equals("imagelab.ImageFilter");
							}//for j
							if (isFilter) {
								obj = cl.newInstance();
								//System.out.println("This is the one: " + fil[k].getName());
								if (obj != null) filters.add(obj);
								JMenuItem jmi = new JMenuItem(((ImageFilter)obj).getMenuLabel());
								filter.add(jmi);
								jmi.addActionListener(makeActionListener(
													(ImageFilter)(filters.lastElement())));
							} //if
						} catch (Exception bigEx) {
							System.err.println("Error in ImageProvider: " + k);
							System.err.println(">>> " + bigEx);
						}//catch
					}//if
	
				}//for k  

		return mbar;
	}//buildMenus

	/**Builds a dedicated actionListener for the specific ImageFilter passed in. */
	public ActionListener makeActionListener (ImageFilter imf) {
		final ImageFilter theFilter = imf;
		return new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
			if (impro == null) return;
			//System.out.println("Using impro number " + impro.getid());
			theFilter.filter(impro);
			impro = theFilter.getImgProvider();
			impro.setLab(theLab);
			images.add(impro);
		}};
	}//makeActionListener


	public ActionListener makeOpenListener() {		//creates action listener for opening a file
		return new ActionListener() {		
			public void actionPerformed(ActionEvent e) {
				ImgProvider improvider;					//an imgProvider to hold the image
			
				File parDir = new File(".");			//set to current directory
				String imdir = "";						//holds image directory
				try {
					imdir = parDir.getCanonicalPath() +  "/images";
					//System.out.println("\n\nParent directory:  " + imdir);
				} catch (Exception excer) {
				}
				//JFileChooser fc = new JFileChooser(new File("images"));
				JFileChooser fc = new JFileChooser();
				//System.out.println("\n\nopen directory --> " + imdir);
				fc.setCurrentDirectory(new File(imdir));
				int retval = fc.showOpenDialog(null);
				File theFile = fc.getSelectedFile();
				String path = theFile.getAbsolutePath();
				//System.out.println("The file's name is " + path);
				//System.out.println("The file's name is " + theDir + theFile);
				improvider = new ImgProvider(path);
				improvider.setLab(theLab);
				//short [][] img = impro.getImage();
				//System.out.println("ImageLab:makeOpenListener - before showImage");
				//improvider.showImage(theDir + theFile); 
				improvider.showImage(path); 
				//System.out.println("ImageLab:makeOpenListener - after showImage");
				images.add(improvider);
				impro = improvider;				//current image provider is set
			}//actionPerformed
		};
	}// makeOpenListener
	

/*
	public ActionListener makeOpenListener() {		//creates action listener for opening a file
		return new ActionListener() {		
			public void actionPerformed(ActionEvent e) {
				ImgProvider improvider;					//an imgProvider to hold the image
				FileDialog fd;
				fd = new FileDialog(frame,"Pick an image",FileDialog.LOAD);
				fd.show();
				String theFile = fd.getFile();
				String theDir = fd.getDirectory();
				//System.out.println("The file's name is " + theDir + theFile);
				improvider = new ImgProvider(theDir + theFile);
				improvider.setLab(theLab);
				//short [][] img = impro.getImage();
				//System.out.println("ImageLab:makeOpenListener - before showImage");
				improvider.showImage(theDir + theFile); 
				//System.out.println("ImageLab:makeOpenListener - after showImage");
				images.add(improvider);
				impro = improvider;				//current image provider is set
			}//actionPerformed
		};
	}// makeOpenListener
*/
	public ActionListener makeSaveListener() {		//creates action listener for saving a file
		return new ActionListener() {		
			public void actionPerformed(ActionEvent e) {
				saver.showSaveDialog(frame);			//return value ignored for now
				String theFile = saver.getName();
				String theDir = saver.getCurrentDirectory().getName();
				File fil = saver.getSelectedFile();
				try {
					impro.save(fil);
				} catch (IOException ioex) {
					System.err.println("Attemt to save image in " + theDir + theFile + " FAILED");
				}
			}//actionPerformed
		};
	}// makeOpenListener
	/**	Marks an image as the one in focus.
	@param ip The ImgProvider responible for the image
	*/
	public void setActive(ImgProvider ip) {
		impro = ip;
		//System.out.println("Setting impro to " + impro.getid());
	}//setActive
}//ImageLab
